import { moveItemInArray, CdkDropList, CdkDrag } from '@angular/cdk/drag-drop';
import * as i6 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Component, ChangeDetectionStrategy } from '@angular/core';
import * as i2 from '@angular/forms';
import { FormsModule } from '@angular/forms';
import { RxVirtualFor, RxVirtualScrollViewportComponent, FixedSizeVirtualScrollStrategy } from '@rx-angular/template/experimental/virtual-scrolling';
import { RxFor } from '@rx-angular/template/for';
import { RxIf } from '@rx-angular/template/if';
import { RxLet } from '@rx-angular/template/let';
import { RxPush } from '@rx-angular/template/push';
import { LoadingElementDirective, FlattenPipe } from '@smallstack/common-components';
import { PdfExportService, PdfExportTableHeaderRenderMode } from '@smallstack/pdf-shared';
import { filterNullish, sortStringWithNumbers, flattenJson, getJsonByPath, convertDotNotationPathToJsonSchemaPath, upperCaseFirst, getLastPropertyName } from '@smallstack/utils';
import { BaseWidgetComponent, WidgetTreeComponent } from '@smallstack/widget-core';
import { BehaviorSubject, map, combineLatest, shareReplay, firstValueFrom } from 'rxjs';

class DataExportWidgetComponent extends BaseWidgetComponent {
    constructor() {
        super(...arguments);
        this.sortBy$ = new BehaviorSubject("createdAt");
        this.sortDirection$ = new BehaviorSubject("asc");
        this.models$ = this.data$.pipe(filterNullish(), map((data) => data.models), filterNullish());
        this.sortedModels$ = combineLatest([this.models$, this.sortBy$, this.sortDirection$]).pipe(map(([models, sortBy, sortDirection]) => {
            if (sortBy === undefined)
                return models;
            return sortStringWithNumbers(models, { direction: sortDirection, subProperty: sortBy });
        }), filterNullish());
        this.schema$ = this.data$.pipe(map((data) => data.schema));
        this.preview$ = this.sortedModels$.pipe(map((models) => {
            return models.slice(0, 25);
        }));
        this.selectedHeaders$ = new BehaviorSubject(undefined);
        this.headers$ = combineLatest([this.models$, this.schema$]).pipe(filterNullish(), map(([models, schema]) => {
            const headers = [];
            models.forEach((model) => {
                const keys = Object.keys(flattenJson(model));
                keys.forEach((key) => {
                    if (headers.find((header) => header.key === key) === undefined) {
                        const schemaForPath = getJsonByPath(schema, convertDotNotationPathToJsonSchemaPath(key));
                        const label = schemaForPath?.["x-schema-form"]?.i18nTitle ||
                            schemaForPath?.title ||
                            upperCaseFirst(getLastPropertyName(key));
                        headers.push({ key, label });
                    }
                });
            });
            if (this.selectedHeaders$.value === undefined)
                this.selectedHeaders$.next(headers.filter((header) => !["id", "createdAt", "lastUpdatedAt"].includes(header.key)).slice(0, 5));
            return headers;
        }), shareReplay(1));
        this.notSelectedHeaders$ = combineLatest([this.headers$, this.selectedHeaders$]).pipe(map(([headers, selectedHeaders]) => headers.filter((header) => !selectedHeaders?.includes(header))), map((headers) => headers.sort()));
        this.format = "pdf";
    }
    setFormat(format) {
        this.format = format;
    }
    close() {
        this.widgetTreeService.dialogRef?.close();
    }
    drop(event) {
        const selectedHeaders = this.selectedHeaders$.value;
        if (event.previousContainer === event.container) {
            moveItemInArray(selectedHeaders, event.previousIndex, event.currentIndex);
        }
        else {
            const header = event.previousContainer.data[event.previousIndex];
            if (selectedHeaders.find((selectedHeader) => header.key === selectedHeader.key) === undefined) {
                selectedHeaders.splice(event.currentIndex, 0, header);
            }
            else {
                selectedHeaders.splice(selectedHeaders.indexOf(header), 1);
            }
        }
        this.selectedHeaders$.next(selectedHeaders);
    }
    async addAll() {
        this.selectedHeaders$.next(await firstValueFrom(this.headers$));
    }
    removeAll() {
        this.selectedHeaders$.next([]);
    }
    add(header) {
        const selectedHeaders = this.selectedHeaders$.value;
        if (selectedHeaders.find((selectedHeader) => header.key === selectedHeader.key) === undefined) {
            selectedHeaders.push(header);
            this.selectedHeaders$.next(selectedHeaders);
        }
    }
    remove(header) {
        const selectedHeaders = this.selectedHeaders$.value;
        selectedHeaders.splice(selectedHeaders.indexOf(header), 1);
        this.selectedHeaders$.next(selectedHeaders);
    }
    export() {
        return async () => {
            const models = await firstValueFrom(this.sortedModels$);
            const headers = await firstValueFrom(this.selectedHeaders$);
            switch (this.format) {
                case "json": {
                    const blob = new Blob([JSON.stringify(models, undefined, 2)], { type: "application/json" });
                    const url = URL.createObjectURL(blob);
                    const a = document.createElement("a");
                    a.href = url;
                    a.download = `data-export.${this.format}`;
                    a.click();
                    break;
                }
                case "pdf": {
                    const pdfExportService = new PdfExportService();
                    const pdf = await pdfExportService.createPdf({
                        data: [
                            {
                                tableHeaders: headers.map((header) => ({
                                    name: header.key,
                                    label: header.label,
                                    renderMode: PdfExportTableHeaderRenderMode.TEXT
                                })),
                                type: "table",
                                values: models
                            }
                        ]
                    });
                    const blob = new Blob([pdf], { type: "application/pdf" });
                    const url = URL.createObjectURL(blob);
                    const a = document.createElement("a");
                    a.href = url;
                    a.download = `data-export.${this.format}`;
                    a.click();
                    break;
                }
                case "csv": {
                    const csvContent = "sep=,\r\n" +
                        headers.map((header) => header.label).join(",") +
                        "\n" +
                        models.map((row) => headers.map((fieldName) => row[fieldName.key]).join(",")).join("\n");
                    const blob = new Blob([new Uint8Array([0xef, 0xbb, 0xbf]), csvContent], {
                        type: "text/csv;charset=utf-8"
                    });
                    const url = URL.createObjectURL(blob);
                    const a = document.createElement("a");
                    a.href = url;
                    a.download = `data-export.${this.format}`;
                    a.click();
                    break;
                }
            }
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.8", ngImport: i0, type: DataExportWidgetComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.2.8", type: DataExportWidgetComponent, isStandalone: true, selector: "smallstack-data-export-widget", usesInheritance: true, ngImport: i0, template: "<ng-container *rxLet=\"selectedHeaders$; let selectedHeaders\">\n  <div class=\"dialog-title\"> Daten exportieren </div>\n  <div class=\"dialog-content p-4 flex flex-col gap-8\">\n    <div class=\"card card-bordered shadow card-compact bg-white\">\n      <div class=\"card-body\">\n        <div class=\"card-title\"> Eigenschaften </div>\n        <div class=\"text-help\"\n          >W\u00E4hlen Sie hier die Eigenschaften / Spalten aus, welche exportiert werden sollen. F\u00FCr den PDF Export ist es\n          hilfreich f\u00FCr eine gute Darstellung, die Spalten auf 5 zu begrenzen.</div\n        >\n        <div class=\"flex flex-row justify-start gap-2 mt-2\">\n          <div class=\"w-1/2 border p-2 rounded-box\">\n            <div class=\"flex flex-row justify-between\">\n              <div class=\"font-bold\">Verf\u00FCgbare Spalten</div>\n              <div>\n                <button class=\"btn btn-xs btn-primary\" (click)=\"addAll()\">Alle hinzuf\u00FCgen</button>\n              </div>\n            </div>\n            <div\n              cdkDropList\n              #availableList=\"cdkDropList\"\n              [cdkDropListData]=\"notSelectedHeaders$ | push\"\n              [cdkDropListConnectedTo]=\"[exportList]\"\n              (cdkDropListDropped)=\"drop($event)\"\n              class=\"h-64 overflow-auto\"\n            >\n              <div\n                class=\"m-2 p-2 rounded-box border cursor-grab flex flex-row justify-between\"\n                *rxFor=\"let header of notSelectedHeaders$\"\n                cdkDrag\n                ><div\n                  >{{ header.label }} <span class=\"variable\">{{ header.key }}</span></div\n                >\n                <button class=\"btn btn-sm btn-primary btn-outline\" (click)=\"add(header)\">+</button>\n              </div>\n            </div>\n          </div>\n\n          <div class=\"w-1/2 border p-2 rounded-box\">\n            <div class=\"flex flex-row justify-between\">\n              <div class=\"font-bold\">Exportierte Spalten</div>\n              <div>\n                <button class=\"btn btn-xs btn-primary\" (click)=\"removeAll()\">Alle entfernen</button>\n              </div>\n            </div>\n            <div\n              cdkDropList\n              #exportList=\"cdkDropList\"\n              [cdkDropListData]=\"selectedHeaders\"\n              [cdkDropListConnectedTo]=\"[availableList]\"\n              (cdkDropListDropped)=\"drop($event)\"\n              class=\"h-64 overflow-auto\"\n            >\n              <div\n                class=\"m-2 p-2 rounded-box border cursor-grab flex flex-row justify-between\"\n                *rxFor=\"let header of selectedHeaders\"\n                cdkDrag\n                ><div\n                  >{{ header.label }} <span class=\"variable\">{{ header.key }}</span></div\n                >\n                <button class=\"btn btn-sm btn-error btn-outline\" (click)=\"remove(header)\">-</button>\n              </div>\n            </div>\n          </div>\n        </div>\n      </div>\n    </div>\n\n    <div class=\"flex flex-row gap-2 w-full justify-evenly\">\n      <div class=\"card card-bordered shadow card-compact bg-white w-full\">\n        <div class=\"card-body\">\n          <div class=\"card-title\"> Sortierung </div>\n          <div class=\"text-help\">W\u00E4hlen Sie hier die Sortierung der Daten aus.</div>\n          <div class=\"flex flex-row gap-2\">\n            <select\n              class=\"select select-bordered w-1/2 max-w-xs\"\n              [ngModel]=\"sortBy$ | push\"\n              (ngModelChange)=\"sortBy$.next($event)\"\n            >\n              <option *rxFor=\"let header of headers$\" [value]=\"header.key\">{{ header.label }}</option>\n            </select>\n            <select\n              class=\"select select-bordered w-1/2 max-w-xs\"\n              [ngModel]=\"sortDirection$ | push\"\n              (ngModelChange)=\"sortDirection$.next($event)\"\n            >\n              <option value=\"asc\">Aufsteigend</option>\n              <option value=\"desc\">Absteigend</option>\n            </select>\n          </div>\n        </div>\n      </div>\n\n      <div class=\"card card-bordered shadow card-compact bg-white w-full\">\n        <div class=\"card-body\">\n          <div class=\"card-title\"> Format Auswahl </div>\n          <div class=\"flex flex-row flex-wrap gap-2\">\n            <div class=\"btn\" (click)=\"setFormat('pdf')\" [ngClass]=\"{ 'btn-primary': format === 'pdf' }\">PDF</div>\n            <div class=\"btn\" (click)=\"setFormat('csv')\" [ngClass]=\"{ 'btn-primary': format === 'csv' }\">CSV</div>\n            <div class=\"btn\" (click)=\"setFormat('json')\" [ngClass]=\"{ 'btn-primary': format === 'json' }\">JSON</div>\n          </div>\n        </div>\n      </div>\n    </div>\n\n    <div class=\"card card-bordered shadow card-compact bg-white\">\n      <div class=\"card-body\">\n        <div class=\"card-title\"> Daten Vorschau </div>\n        <div class=\"text-help\">Hier sehen Sie eine Vorschau der Daten, welche exportiert werden.</div>\n        <div class=\"overflow-auto w-full whitespace-nowrap\">\n          <table class=\"table table-zebra w-full table-border\">\n            <thead>\n              <tr>\n                <th *rxFor=\"let header of selectedHeaders\">{{ header.label }}</th>\n              </tr>\n            </thead>\n            <tbody>\n              <tr *rxFor=\"let model of preview$\">\n                <td *rxFor=\"let header of selectedHeaders\">{{ model | flatten: header.key }}</td>\n              </tr>\n            </tbody>\n          </table>\n        </div>\n        <div *rxIf=\"(preview$ | push).length < (models$ | push).length\" class=\"text-help text-center\"\n          >Es werden {{ (preview$ | push).length }} von {{ (models$ | push).length }} Datens\u00E4tze(n) angezeigt.</div\n        >\n      </div>\n    </div>\n  </div>\n  <div class=\"dialog-actions\">\n    <button (click)=\"close()\" class=\"btn btn-sm\">Abbrechen</button>\n    <button [loadingFn]=\"export()\" class=\"btn btn-sm btn-primary\">Exportieren</button>\n  </div>\n</ng-container>\n", dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i6.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "ngmodule", type: FormsModule }, { kind: "directive", type: i2.NgSelectOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.ɵNgSelectMultipleOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.SelectControlValueAccessor, selector: "select:not([multiple])[formControlName],select:not([multiple])[formControl],select:not([multiple])[ngModel]", inputs: ["compareWith"] }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }, { kind: "pipe", type: RxPush, name: "push" }, { kind: "directive", type: RxFor, selector: "[rxFor][rxForOf]", inputs: ["rxForOf", "rxForTemplate", "rxForStrategy", "rxForParent", "rxForPatchZone", "rxForTrackBy", "rxForRenderCallback"] }, { kind: "directive", type: RxLet, selector: "[rxLet]", inputs: ["rxLet", "rxLetStrategy", "rxLetComplete", "rxLetError", "rxLetSuspense", "rxLetContextTrigger", "rxLetCompleteTrigger", "rxLetErrorTrigger", "rxLetSuspenseTrigger", "rxLetNextTrigger", "rxLetRenderCallback", "rxLetParent", "rxLetPatchZone"], outputs: ["rendered"] }, { kind: "directive", type: RxIf, selector: "[rxIf]", inputs: ["rxIf", "rxIfStrategy", "rxIfElse", "rxIfThen", "rxIfSuspense", "rxIfComplete", "rxIfError", "rxIfContextTrigger", "rxIfNextTrigger", "rxIfSuspenseTrigger", "rxIfErrorTrigger", "rxIfCompleteTrigger", "rxIfParent", "rxIfPatchZone", "rxIfRenderCallback"] }, { kind: "directive", type: LoadingElementDirective, selector: "[loadingFn]", inputs: ["loadingFn", "isLoading"], exportAs: ["loadingFn"] }, { kind: "directive", type: CdkDropList, selector: "[cdkDropList], cdk-drop-list", inputs: ["cdkDropListConnectedTo", "cdkDropListData", "cdkDropListOrientation", "id", "cdkDropListLockAxis", "cdkDropListDisabled", "cdkDropListSortingDisabled", "cdkDropListEnterPredicate", "cdkDropListSortPredicate", "cdkDropListAutoScrollDisabled", "cdkDropListAutoScrollStep"], outputs: ["cdkDropListDropped", "cdkDropListEntered", "cdkDropListExited", "cdkDropListSorted"], exportAs: ["cdkDropList"] }, { kind: "directive", type: CdkDrag, selector: "[cdkDrag]", inputs: ["cdkDragData", "cdkDragLockAxis", "cdkDragRootElement", "cdkDragBoundary", "cdkDragStartDelay", "cdkDragFreeDragPosition", "cdkDragDisabled", "cdkDragConstrainPosition", "cdkDragPreviewClass", "cdkDragPreviewContainer"], outputs: ["cdkDragStarted", "cdkDragReleased", "cdkDragEnded", "cdkDragEntered", "cdkDragExited", "cdkDragDropped", "cdkDragMoved"], exportAs: ["cdkDrag"] }, { kind: "pipe", type: FlattenPipe, name: "flatten" }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.8", ngImport: i0, type: DataExportWidgetComponent, decorators: [{
            type: Component,
            args: [{ selector: "smallstack-data-export-widget", standalone: true, imports: [
                        CommonModule,
                        FormsModule,
                        RxPush,
                        RxVirtualFor,
                        RxFor,
                        RxLet,
                        RxIf,
                        RxVirtualScrollViewportComponent,
                        FixedSizeVirtualScrollStrategy,
                        WidgetTreeComponent,
                        LoadingElementDirective,
                        CdkDropList,
                        CdkDrag,
                        FlattenPipe
                    ], changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-container *rxLet=\"selectedHeaders$; let selectedHeaders\">\n  <div class=\"dialog-title\"> Daten exportieren </div>\n  <div class=\"dialog-content p-4 flex flex-col gap-8\">\n    <div class=\"card card-bordered shadow card-compact bg-white\">\n      <div class=\"card-body\">\n        <div class=\"card-title\"> Eigenschaften </div>\n        <div class=\"text-help\"\n          >W\u00E4hlen Sie hier die Eigenschaften / Spalten aus, welche exportiert werden sollen. F\u00FCr den PDF Export ist es\n          hilfreich f\u00FCr eine gute Darstellung, die Spalten auf 5 zu begrenzen.</div\n        >\n        <div class=\"flex flex-row justify-start gap-2 mt-2\">\n          <div class=\"w-1/2 border p-2 rounded-box\">\n            <div class=\"flex flex-row justify-between\">\n              <div class=\"font-bold\">Verf\u00FCgbare Spalten</div>\n              <div>\n                <button class=\"btn btn-xs btn-primary\" (click)=\"addAll()\">Alle hinzuf\u00FCgen</button>\n              </div>\n            </div>\n            <div\n              cdkDropList\n              #availableList=\"cdkDropList\"\n              [cdkDropListData]=\"notSelectedHeaders$ | push\"\n              [cdkDropListConnectedTo]=\"[exportList]\"\n              (cdkDropListDropped)=\"drop($event)\"\n              class=\"h-64 overflow-auto\"\n            >\n              <div\n                class=\"m-2 p-2 rounded-box border cursor-grab flex flex-row justify-between\"\n                *rxFor=\"let header of notSelectedHeaders$\"\n                cdkDrag\n                ><div\n                  >{{ header.label }} <span class=\"variable\">{{ header.key }}</span></div\n                >\n                <button class=\"btn btn-sm btn-primary btn-outline\" (click)=\"add(header)\">+</button>\n              </div>\n            </div>\n          </div>\n\n          <div class=\"w-1/2 border p-2 rounded-box\">\n            <div class=\"flex flex-row justify-between\">\n              <div class=\"font-bold\">Exportierte Spalten</div>\n              <div>\n                <button class=\"btn btn-xs btn-primary\" (click)=\"removeAll()\">Alle entfernen</button>\n              </div>\n            </div>\n            <div\n              cdkDropList\n              #exportList=\"cdkDropList\"\n              [cdkDropListData]=\"selectedHeaders\"\n              [cdkDropListConnectedTo]=\"[availableList]\"\n              (cdkDropListDropped)=\"drop($event)\"\n              class=\"h-64 overflow-auto\"\n            >\n              <div\n                class=\"m-2 p-2 rounded-box border cursor-grab flex flex-row justify-between\"\n                *rxFor=\"let header of selectedHeaders\"\n                cdkDrag\n                ><div\n                  >{{ header.label }} <span class=\"variable\">{{ header.key }}</span></div\n                >\n                <button class=\"btn btn-sm btn-error btn-outline\" (click)=\"remove(header)\">-</button>\n              </div>\n            </div>\n          </div>\n        </div>\n      </div>\n    </div>\n\n    <div class=\"flex flex-row gap-2 w-full justify-evenly\">\n      <div class=\"card card-bordered shadow card-compact bg-white w-full\">\n        <div class=\"card-body\">\n          <div class=\"card-title\"> Sortierung </div>\n          <div class=\"text-help\">W\u00E4hlen Sie hier die Sortierung der Daten aus.</div>\n          <div class=\"flex flex-row gap-2\">\n            <select\n              class=\"select select-bordered w-1/2 max-w-xs\"\n              [ngModel]=\"sortBy$ | push\"\n              (ngModelChange)=\"sortBy$.next($event)\"\n            >\n              <option *rxFor=\"let header of headers$\" [value]=\"header.key\">{{ header.label }}</option>\n            </select>\n            <select\n              class=\"select select-bordered w-1/2 max-w-xs\"\n              [ngModel]=\"sortDirection$ | push\"\n              (ngModelChange)=\"sortDirection$.next($event)\"\n            >\n              <option value=\"asc\">Aufsteigend</option>\n              <option value=\"desc\">Absteigend</option>\n            </select>\n          </div>\n        </div>\n      </div>\n\n      <div class=\"card card-bordered shadow card-compact bg-white w-full\">\n        <div class=\"card-body\">\n          <div class=\"card-title\"> Format Auswahl </div>\n          <div class=\"flex flex-row flex-wrap gap-2\">\n            <div class=\"btn\" (click)=\"setFormat('pdf')\" [ngClass]=\"{ 'btn-primary': format === 'pdf' }\">PDF</div>\n            <div class=\"btn\" (click)=\"setFormat('csv')\" [ngClass]=\"{ 'btn-primary': format === 'csv' }\">CSV</div>\n            <div class=\"btn\" (click)=\"setFormat('json')\" [ngClass]=\"{ 'btn-primary': format === 'json' }\">JSON</div>\n          </div>\n        </div>\n      </div>\n    </div>\n\n    <div class=\"card card-bordered shadow card-compact bg-white\">\n      <div class=\"card-body\">\n        <div class=\"card-title\"> Daten Vorschau </div>\n        <div class=\"text-help\">Hier sehen Sie eine Vorschau der Daten, welche exportiert werden.</div>\n        <div class=\"overflow-auto w-full whitespace-nowrap\">\n          <table class=\"table table-zebra w-full table-border\">\n            <thead>\n              <tr>\n                <th *rxFor=\"let header of selectedHeaders\">{{ header.label }}</th>\n              </tr>\n            </thead>\n            <tbody>\n              <tr *rxFor=\"let model of preview$\">\n                <td *rxFor=\"let header of selectedHeaders\">{{ model | flatten: header.key }}</td>\n              </tr>\n            </tbody>\n          </table>\n        </div>\n        <div *rxIf=\"(preview$ | push).length < (models$ | push).length\" class=\"text-help text-center\"\n          >Es werden {{ (preview$ | push).length }} von {{ (models$ | push).length }} Datens\u00E4tze(n) angezeigt.</div\n        >\n      </div>\n    </div>\n  </div>\n  <div class=\"dialog-actions\">\n    <button (click)=\"close()\" class=\"btn btn-sm\">Abbrechen</button>\n    <button [loadingFn]=\"export()\" class=\"btn btn-sm btn-primary\">Exportieren</button>\n  </div>\n</ng-container>\n" }]
        }] });

export { DataExportWidgetComponent };
//# sourceMappingURL=smallstack-data-widgets-data-export-widget.component-f46aaea8.mjs.map
